# Copyright (C) 2025  Andreas Delabie

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.



import os, sysconfig, darkdetect



def add():
    '''Adds the Init-Film shortcut to the Windows context menu.'''


    osname = os.name
    if osname != 'nt':
        print('This feature currently only works on Windows!')
        exit()
    else:
        import winreg


    try:
        python_scripts = sysconfig.get_path('scripts')
        python_sitepackages = sysconfig.get_path('purelib')
        windows_theme = darkdetect.theme()

        location = winreg.HKEY_CURRENT_USER
        shell = winreg.CreateKey(location, 'Software\\Classes\\Directory\\Background\\shell')
        initfilm = winreg.CreateKey(shell, 'Init-Film')
        command = winreg.CreateKey(initfilm, 'command')

        winreg.SetValueEx(initfilm, '', 0, winreg.REG_SZ, 'Start Init-Film Project') # Shortcut name
        winreg.SetValueEx(command, '', 0, winreg.REG_SZ, f'{python_scripts}/init-film.exe') # Shortcut executable path

        if windows_theme == 'Dark':
            winreg.SetValueEx(initfilm, 'Icon', 0, winreg.REG_SZ, f'{python_sitepackages}/initfilm/Init-Film_Icon_White.ico')
        else:
            winreg.SetValueEx(initfilm, 'Icon', 0, winreg.REG_SZ, f'{python_sitepackages}/initfilm/Init-Film_Icon_Black.ico')
        
        print('Successfully added Init-Film to context menu!')


    except Exception as err:
        if str(err).__contains__('WinError 5'):
            print(err, '\n\nAre you administrator?')
        else:
            print(err)



def remove():
    '''Removes the Init-Film shortcut to the Windows context menu.'''

    
    osname = os.name
    if osname != 'nt':
        print('This feature currently only works on Windows!')
        exit()
    else:
        import winreg


    try:
        location = winreg.HKEY_CURRENT_USER
        shell = winreg.CreateKey(location, 'Software\\Classes\\Directory\\Background\\shell')
        initfilm = winreg.CreateKey(shell, 'Init-Film')
        command = winreg.CreateKey(initfilm, 'command')

        winreg.DeleteValue(command, '')
        winreg.DeleteValue(initfilm, '')
        winreg.DeleteValue(initfilm, 'Icon')

        winreg.DeleteKey(command, '')
        winreg.CloseKey(command)
        winreg.DeleteKey(initfilm, '')
        winreg.CloseKey(initfilm)
        
        print('Successfully removed Init-Film from context menu!')


    except Exception as err:
        if str(err).__contains__('WinError 5'):
            print(err, '\n\nAre you administrator?')
        elif str(err).__contains__('WinError 2'):
            print(err, '\n\nThis probably means the shortcut has already been removed, or hasn\'t been installed correctly...')
        else:
            print(err)


# Copyright (C) 2025  Andreas Delabie

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.



import os, sysconfig, shutil, yaspin
from . import config
from .clearconsole import clearConsole



python_sitepackages = sysconfig.get_path('purelib')
template_path = config.get('templates', 'path')

if template_path == 'python':
    template_path = f'{python_sitepackages}/initfilm/templates'
    os.makedirs(template_path, exist_ok=True)
else:
    os.makedirs(template_path, exist_ok=True)



def check(relative_source_path:str) -> bool:
    '''Checks if source folder contains template files.
    Args:
        relative_source_path (str): Path to template files, relative to `template_path` (ex. 'ASSETS/LOGOS')
    Returns:
        result (bool): `True` if folder contains files, `False` if it doesn't.'''

    source_path = os.path.join(template_path, relative_source_path)

    try:
        files = [f for f in os.listdir(source_path) if os.path.isfile(os.path.join(source_path, f)) and f != '.DS_Store'] # Lists all contents in source_path & excludes folders and .DS_Store files

        if files == []:
            return False
        else:
            return True

    except:
        return False



def copy(relative_source_path:str, destination_path:str, destination_name:str):
    '''Lists all template files in source folder, let's user select files & copies files to destination.
    Args:
        relative_source_path (str): Path to template files, relative to `template_path` (ex. 'ASSETS/LOGOS').
        destination_path (str): Full path to destination folder.
        destination_name (str): Name of the destination folder, used in the selection screen (ex. 1. PROJECT FILES).
    '''

    source_path = os.path.join(template_path, relative_source_path)
    files = [f for f in os.listdir(source_path) if os.path.isfile(os.path.join(source_path, f)) and f != '.DS_Store'] # Lists all contents in source_path & excludes folders and .DS_Store files

    clearConsole()
    print(f'Select needed files for {destination_name}\n')
    for id, template in enumerate(files, start=1):
        print(f'    {id}) {template}')
    print('\nType numbers to select (separate by SPACE)')
    selection = input('$ ').split()

    for id in selection:
        template = files[int(id) - 1]

        with yaspin.yaspin(color='yellow', text=f'Copying {template} to {destination_name}...') as spinner:
            shutil.copy2(f'{source_path}/{template}', destination_path)
            spinner.text = f'Copying {template} to {destination_name} completed!'
            spinner.ok('✔')


# Copyright (C) 2025  Andreas Delabie

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.



import typer, initfilm
from typing_extensions import Annotated
from . import main, config, transcode, ffmpeg_installscript, license



app = typer.Typer(add_completion=False, context_settings={'help_option_names': ['--help', '-h']})
app.add_typer(license.app, name='license', help='Show license information.')
app.add_typer(config.app, name='config', help='Manage configuration settings.')
def cli():
    app()



@app.callback(invoke_without_command=True)
def default(context: typer.Context):
    if context.invoked_subcommand is None: # Only run if no subcommand is provided
        main.createFolder_level0()

@app.command()
def create_proxies(
    folder_footage:Annotated[str, typer.Argument(help="Full path to footage folder. (Use '.' for current working directory)")],
    codec:Annotated[str, typer.Option('--codec', '-c', help="Use 'h264', 'h264-nvidia', 'h264-amd', 'h264-intel', 'dnxhr', 'prores-proxy' or 'prores-lt'")]=transcode.detect_defaults('codec'),
    resolution:Annotated[str, typer.Option('--resolution', '-r', help="Ex. '1280x720', '1920x1080', '3840x2160', ...")]=transcode.detect_defaults('resolution')):
    '''Create proxies for footage in the specified footage folder.'''
    transcode.create_proxies(folder_footage, codec, resolution)

@app.command()
def install_ffmpeg():
    '''Install FFmpeg (needed to transcode footage).'''
    ffmpeg_installscript.main()

@app.command()
def version():
    '''Print the current version.'''
    version = initfilm.__version__
    print(version)


import typer



app = typer.Typer(add_completion=False)



@app.callback(invoke_without_command=True)
def show_license(context:typer.Context):
    '''Show license information.'''
    if context.invoked_subcommand is None:  # Only run if no subcommand is provided
        print('''
Copyright (C) 2025  Andreas Delabie

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
''')



@app.command(name='warranty')
def show_warranty():
    '''Show warranty information.'''
    print('''
-------------------------------------
Disclaimer of Warranty.
-------------------------------------

THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY
APPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY
OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM
IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF
ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

-------------------------------------
Limitation of Liability.
-------------------------------------

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS
THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),
EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

-------------------------------------
Interpretation of Sections 15 and 16.
-------------------------------------

If the disclaimer of warranty and limitation of liability provided
above cannot be given local legal effect according to their terms,
reviewing courts shall apply local law that most closely approximates
an absolute waiver of all civil liability in connection with the
Program, unless a warranty or assumption of liability accompanies a
copy of the Program in return for a fee.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
''')



@app.command(name='conditions')
def show_conditions():
    '''Show conditions information.'''
    print('''
-----------------------------------
Conveying Modified Source Versions.
-----------------------------------

You may convey a work based on the Program, or the modifications to
produce it from the Program, in the form of source code under the
terms of section 4, provided that you also meet all of these conditions:

    a) The work must carry prominent notices stating that you modified
    it, and giving a relevant date.

    b) The work must carry prominent notices stating that it is
    released under this License and any conditions added under section
    7.  This requirement modifies the requirement in section 4 to
    "keep intact all notices".

    c) You must license the entire work, as a whole, under this
    License to anyone who comes into possession of a copy.  This
    License will therefore apply, along with any applicable section 7
    additional terms, to the whole of the work, and all its parts,
    regardless of how they are packaged.  This License gives no
    permission to license the work in any other way, but it does not
    invalidate such permission if you have separately received it.

    d) If the work has interactive user interfaces, each must display
    Appropriate Legal Notices; however, if the Program has interactive
    interfaces that do not display Appropriate Legal Notices, your
    work need not make them do so.

A compilation of a covered work with other separate and independent
works, which are not by their nature extensions of the covered work,
and which are not combined with it such as to form a larger program,
in or on a volume of a storage or distribution medium, is called an
"aggregate" if the compilation and its resulting copyright are not
used to limit the access or legal rights of the compilation's users
beyond what the individual works permit.  Inclusion of a covered work
in an aggregate does not cause this License to apply to the other
parts of the aggregate.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
''')


# Copyright (C) 2025  Andreas Delabie

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.



import os, pathlib, re, platform, subprocess, sysconfig, json
from . import config
from .clearconsole import clearConsole



PLATFORM = platform.system().lower()



def find_folder(folder_footage:str, subfolder:str) -> str:
    '''Auto-detect RAW and PROXIES folders in footage folder.
    Args:
        folder_footage (str): Full path to footage folder.
        subfolder (str): Name of the subfolder to find (ex. 'RAW')
    Returns:
        match (str): Matching folder path.'''

    matches = []

    for folder in os.listdir(folder_footage):
        if re.match(f'(\d|\d\d)(.\s|\s){subfolder}', folder): # Match folder names like '01 RAW', '2. PROXIES', etc.
            matches.append(os.path.join(folder_footage, folder))

    if not matches:
        raise FileNotFoundError(f"No folder named '{subfolder}' found in {folder_footage}")

    return matches[0]



def detect_defaults(returns:str) -> str:
    '''Detect default settings for transcoding. Set defaults if not already set.
    Args:
        returns (str): The setting to return (ex. 'codec', 'resolution').
    Returns:
        value (str): The value of the requested setting.'''

    global default_codec, default_resolution

    try:
        default_codec = config.get('proxies', 'default_codec')
        default_resolution = config.get('proxies', 'default_resolution')
    except:
        config.set_proxy_codec('h264')
        config.set_proxy_resolution('1280x720')
        default_codec = config.get('proxies', 'default_codec')
        default_resolution = config.get('proxies', 'default_resolution')

    match returns:
        case 'codec':
            return default_codec
        case 'resolution':
            return default_resolution
        


def get_preset(codec:str) -> str:
    '''Get the FFmpeg preset for the given codec and platform.
    Args:
        codec (str): The codec to get the preset for.
    Returns:
        preset (str): The FFmpeg preset for the given codec and platform. **Needs to be formatted!**  
        ex: `get_preset('h264').format(file_input=os.path.join(folder_raw,file))`'''
    
    python_sitepackages = sysconfig.get_path('purelib')

    with open(f'{python_sitepackages}/initfilm/ffmpeg_presets.json', 'r') as file:
        presets = json.load(file)

    try:
        return presets[codec][PLATFORM]
    except KeyError:
        try:
            return presets[codec]['generic']
        except KeyError:
            raise NotImplementedError (f"Codec '{codec}' is not supported on your platform ({PLATFORM})")



def transcode(folder_raw:str, folder_proxies:str, codec:str=detect_defaults('codec'), resolution:str=detect_defaults('resolution')):
    '''Transcode all video files in RAW folder to PROXIES folder.
    Args:
        folder_raw (str): Full path to RAW folder.
        folder_proxies (str): Full path to PROXIES folder.
        codec (str): Codec to use for transcoding (default: 'h264').
        resolution (str): Resolution to use for transcoding (default: '1280x720').'''
    
    for file in os.listdir(folder_raw):
        if not file.lower().endswith(('.mp4', '.mov', '.avi', '.mts', '.mxf', '.mkv', '.wmv', '.flv')): # Only process video files
            continue

        preset = get_preset(codec).format(
            file_input = os.path.join(folder_raw, file),
            file_output = f'{os.path.join(folder_proxies, pathlib.Path(file).stem)}_proxy_{codec}_{resolution}',
            resolution=resolution,
            w = resolution.split('x')[0],
            h = resolution.split('x')[1],
            threads=os.cpu_count()
        )

        print(f'Transcoding {file} to {codec} {resolution}...')
        subprocess.run(preset, shell=True, check=True)



def create_proxies(folder_footage:str, codec:str=detect_defaults('codec'), resolution:str=detect_defaults('resolution')):
    '''Find RAW and PROXIES folders in FOOTAGE folder and create proxies.
    Args:
        folder_footage (str): Full path to footage folder. (Use '.' for current working directory)
        codec (str): Codec to use for transcoding (default: 'h264').
        resolution (str): Resolution to use for transcoding (default: '1280x720').'''

    if folder_footage == '.':
        folder_footage = os.getcwd()
    folder_raw = find_folder(folder_footage, 'RAW')
    subfolders_raw = sorted([f for f in os.listdir(folder_raw) if os.path.isdir(os.path.join(folder_raw, f))])
    folder_proxies = find_folder(folder_footage, 'PROXIES')

    if subfolders_raw:
        clearConsole()
        print(f'Select folders to transcode:\n')
        for id, folder in enumerate(subfolders_raw, start=1):
            print(f'    {id}) {folder}')
        print('\nType numbers to select (separate by SPACE)')
        folder_selection = input('$ ').split()

        for id in folder_selection:
            folder_input = subfolders_raw[int(id) - 1]
            folder_input = os.path.join(folder_raw, folder_input)

            transcode(folder_input, folder_proxies, codec, resolution)
    else:
        transcode(folder_raw, folder_proxies, codec, resolution)



# Copyright (C) 2025  Andreas Delabie

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.



import os, pyperclip, initfilm
from . import config, templates, transcode
from .clearconsole import clearConsole



def prefix(number_input:int) -> str:
    '''Returns a styled prefix based on the values in the config file.
    Args:
        number_input (int): The prefix number.
    Returns:
        prefix (str): Styled prefix with a number and separator.'''

    try:
        visible = config.get('prefix', 'visibility')
    except:
        config.set_prefix_visibility('visible')
        visible = config.get('prefix', 'visibility')
    number_style = config.get('prefix', 'number_style')
    separator_style = config.get('prefix', 'separator_style')

    if number_style == 'double':
        number_output = f'{number_input:02d}'
    else:
        number_output = number_input
    
    match separator_style:
        case 'underscore':
            separator = '_'
        case 'parenthesis':
            separator = ') '
        case 'space':
            separator = ' '
        case _:
            separator = '. '
    
    if visible == 'hidden':
        return ''
    else:
        return f'{number_output}{separator}'



def createFolder_level0():
    global projectDirectory, workingTitle

    clearConsole()
    print(f'''Init-Film v{initfilm.__version__}  Copyright (C) 2025  Andreas Delabie
This program comes with ABSOLUTELY NO WARRANTY; for details type `init-film license warranty`.
This is free software, and you are welcome to redistribute it
under certain conditions; type `init-film license conditions` for details.

Enter project name:''')

    workingTitle = input('$ ')
    pyperclip.copy(workingTitle)
    currentDirectory = os.getcwd()
    projectDirectory = os.path.join(currentDirectory, workingTitle)

    os.mkdir(projectDirectory)
    if templates.check('') == True:
        templates.copy('', projectDirectory, workingTitle)

    createFolder_level1_project()

    if proxies_footage:
        createProxies()



def createFolder_level1(number:int, name:str):
    global level1_folder_name
    level1_folder_name = f'{prefix(number)}{name}'
    level1_folder_directory = os.path.join(projectDirectory, level1_folder_name)

    os.mkdir(level1_folder_directory)
    if templates.check(name) == True:
        templates.copy(name, level1_folder_directory, level1_folder_name)



def createFolder_level1_project():
    clearConsole()
    print(f'''Select needed level 1 folders for {workingTitle}

    1) PROJECT FILES
    2) ASSETS
    3) FOOTAGE
    4) AUDIO
    5) EXPORT
    6) DOCUMENTS
    7) MARKETING

Type numbers to select or type custom folder names (separate by SPACE)''')

    level1_project_selection = input('$ ').split()
    global level1_project_folderNumber
    level1_project_folderNumber = 1

    for string in level1_project_selection: 
        match string:
            case '1':
                createFolder_level1(level1_project_folderNumber, 'PROJECT FILES')
                createFolder_level2_projectFiles()
            case '2':
                createFolder_level1(level1_project_folderNumber, 'ASSETS')
                createFolder_level2_assets()
            case '3':
                createFolder_level1(level1_project_folderNumber, 'FOOTAGE')
                createFolder_level2_footage()
            case '4':
                createFolder_level1(level1_project_folderNumber, 'AUDIO')
                createFolder_level2_audio()
            case '5':
                createFolder_level1(level1_project_folderNumber, 'EXPORT')
                createFolder_level2_export()
            case '6':
                createFolder_level1(level1_project_folderNumber, 'DOCUMENTS')
                createFolder_level2_documents()
            case '7':
                createFolder_level1(level1_project_folderNumber, 'MARKETING')
                createFolder_level2_marketing()
            case _:
                createFolder_level1(level1_project_folderNumber, string.upper())
                createFolder_level2_custom(f'{prefix(level1_project_folderNumber)}{string.upper()}', string.upper())
        level1_project_folderNumber = level1_project_folderNumber + 1



def createFolder_level2(parent:str, number:int, name:str, relative_template_path:str):
    global level2_folder_name
    level2_folder_name = f'{prefix(number)}{name}'
    level2_folder_directory = os.path.join(projectDirectory, parent, level2_folder_name)

    os.mkdir(level2_folder_directory)
    if templates.check(relative_template_path) == True:
        templates.copy(relative_template_path, level2_folder_directory, level2_folder_name)



def createFolder_level2_projectFiles():
    clearConsole()
    print(f'''Select needed level 2 folders for {level1_folder_name}:

    EDITING:
        1) PREMIERE PRO
        2) DAVINCI RESOLVE
        3) MEDIA COMPOSER
        4) FINAL CUT PRO
        5) SONY VEGAS
        6) HITFILM EXPRESS

    GRAPHICS/VFX:
        7) AFTER EFFECTS
        8) PHOTOSHOP
        9) ILLUSTRATOR
        10) BLENDER
        11) CINEMA 4D
        12) MAYA
        13) NUKE
        14) HOUDINI
        15) ZBRUSH
        16) AFFINITY DESIGNER

    AUDIO/MUSIC:
        17) PRO TOOLS
        18) ABLETON
        19) CUBASE
        20) REAPER
        21) IZOTOPE RX
        22) AUDITION
        23) AUDACITY
        24) LOGIC PRO
        25) FL STUDIO

Type numbers to select or type custom folder names (separate by SPACE)''')

    level2_projectFiles_selection = input('$ ').split()
    level2_projectFiles_folderNumber = 1

    for string in level2_projectFiles_selection:
        match string:
            case '1': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'PREMIERE PRO', 'PROJECT FILES/PREMIERE PRO')
            case '2': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'DAVINCI RESOLVE', 'PROJECT FILES/DAVINCI RESOLVE')
            case '3': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'MEDIA COMPOSER', 'PROJECT FILES/MEDIA COMPOSER')
            case '4': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'FINAL CUT PRO', 'PROJECT FILES/FINAL CUT PRO')
            case '5': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'SONY VEGAS', 'PROJECT FILES/SONY VEGAS')
            case '6': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'HITFILM EXPRESS', 'PROJECT FILES/HITFILM EXPRESS')
            case '7': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'AFTER EFFECT', 'PROJECT FILES/AFTER EFFECTS')
            case '8': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'PHOTOSHOP', 'PROJECT FILES/PHOTOSHOP')
            case '9': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'ILLUSTRATOR', 'PROJECT FILES/ILLUSTRATOR')
            case '10': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'BLENDER', 'PROJECT FILES/BLENDER')
            case '11': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'CINEMA 4D', 'PROJECT FILES/CINEMA 4D')
            case '12': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'MAYA', 'PROJECT FILES/MAYA')
            case '13': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'NUKE', 'PROJECT FILES/NUKE')
            case '14': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'HOUDINI', 'PROJECT FILES/HOUDINI')
            case '15': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'ZBRUSH', 'PROJECT FILES/ZBRUSH')
            case '16': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'AFFINITY DESIGNER', 'PROJECT FILES/AFFINITY DESIGNER')
            case '17': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'PRO TOOLS', 'PROJECT FILES/PRO TOOLS')
            case '18': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'ABLETON', 'PROJECT FILES/ABLETON')
            case '19': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'CUBASE', 'PROJECT FILES/CUBASE')
            case '20': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'REAPER', 'PROJECT FILES/REAPER')
            case '21': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'IZOTOPE RX', 'PROJECT FILES/IZOTOPE RX')
            case '22': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'AUDITION', 'PROJECT FILES/AUDITION')
            case '23': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'AUDACITY', 'PROJECT FILES/AUDACITY')
            case '24': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'LOGIC PRO', 'PROJECT FILES/LOGIC PRO')
            case '25': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'FL STUDIO', 'PROJECT FILES/FL STUDIO')
            case _: createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, string.upper(), f'PROJECT FILES/{string.upper()}')
        level2_projectFiles_folderNumber = level2_projectFiles_folderNumber + 1



def createFolder_level2_assets():
    clearConsole()
    print(f'''Select needed level 2 folders for {level1_folder_name}

    1. LOGOS
    2. GRAPHICS
    3. FONTS
    4. OVERLAYS
    5. REFERENCE

Type custom folder names (separate by SPACE)''')

    level2_assets_selection = input('$ ').split()
    level2_assets_folderNumber = 1

    for string in level2_assets_selection:
        match string:
            case '1': createFolder_level2(f'{prefix(level1_project_folderNumber)}ASSETS', level2_assets_folderNumber, 'LOGOS', 'ASSETS/LOGOS')
            case '2': createFolder_level2(f'{prefix(level1_project_folderNumber)}ASSETS', level2_assets_folderNumber, 'GRAPHICS', 'ASSETS/GRAPHICS')
            case '3': createFolder_level2(f'{prefix(level1_project_folderNumber)}ASSETS', level2_assets_folderNumber, 'FONTS', 'ASSETS/FONTS')
            case '4': createFolder_level2(f'{prefix(level1_project_folderNumber)}ASSETS', level2_assets_folderNumber, 'OVERLAYS', 'ASSETS/OVERLAYS')
            case '5': createFolder_level2(f'{prefix(level1_project_folderNumber)}ASSETS', level2_assets_folderNumber, 'REFERENCE', 'ASSETS/REFERENCE')
            case _: createFolder_level2(f'{prefix(level1_project_folderNumber)}ASSETS', level2_assets_folderNumber, string.upper(), f'ASSETS/{string.upper()}')
    level2_assets_folderNumber = level2_assets_folderNumber + 1



def createFolder_level2_footage():
    clearConsole()
    print(f'''Select needed level 2 folders for {level1_folder_name}
    
    1) RAW
    2) PROXIES
    3) STOCK
    4) VFX
    5) GRADED
          
Type numbers to select or type custom folder names (separate by SPACE)''')

    level2_footage_selection = input('$ ').split()
    global level2_footage_folderNumber, proxies_footage
    level2_footage_folderNumber = 1
    proxies_footage = None

    for string in level2_footage_selection:
        match string:
            case '1':
                createFolder_level2(f'{prefix(level1_project_folderNumber)}FOOTAGE', level2_footage_folderNumber, 'RAW', 'FOOTAGE/RAW')
                createFolder_level3_raw()
            case '2':
                createFolder_level2(f'{prefix(level1_project_folderNumber)}FOOTAGE', level2_footage_folderNumber, 'PROXIES', 'FOOTAGE/PROXIES')
                proxies_footage = f'{prefix(level1_project_folderNumber)}FOOTAGE'
            case '3': createFolder_level2(f'{prefix(level1_project_folderNumber)}FOOTAGE', level2_footage_folderNumber, 'STOCK', 'FOOTAGE/STOCK')
            case '4': createFolder_level2(f'{prefix(level1_project_folderNumber)}FOOTAGE', level2_footage_folderNumber, 'VFX', 'FOOTAGE/VFX')
            case '5': createFolder_level2(f'{prefix(level1_project_folderNumber)}FOOTAGE', level2_footage_folderNumber, 'GRADED', 'FOOTAGE/GRADED')
            case _: createFolder_level2(f'{prefix(level1_project_folderNumber)}FOOTAGE', level2_footage_folderNumber, string.upper(), f'FOOTAGE/{string.upper()}')
        level2_footage_folderNumber = level2_footage_folderNumber + 1



def createFolder_level2_audio():
    clearConsole()
    print(f'''Select needed level 2 folders for {level1_folder_name}

    1) PFX
    2) SFX
    3) FOLEY
    4) AMBIANCE
    5) MUSIC
    6) SOUNDTRACK
    7) VOICEOVER
    8) ADR

Type numbers to select or type custom folder names (separate by SPACE)''')
    
    level2_audio_selection = input('$ ').split()
    level2_audio_folderNumber = 1

    for string in level2_audio_selection:
        match string:
            case '1': createFolder_level2(f'{prefix(level1_project_folderNumber)}AUDIO', level2_audio_folderNumber, 'PFX', 'AUDIO/PFX')
            case '2': createFolder_level2(f'{prefix(level1_project_folderNumber)}AUDIO', level2_audio_folderNumber, 'SFX', 'AUDIO/SFX')
            case '3': createFolder_level2(f'{prefix(level1_project_folderNumber)}AUDIO', level2_audio_folderNumber, 'FOLEY', 'AUDIO/FOLEY')
            case '4': createFolder_level2(f'{prefix(level1_project_folderNumber)}AUDIO', level2_audio_folderNumber, 'AMBIANCE', 'AUDIO/AMBIANCE')
            case '5': createFolder_level2(f'{prefix(level1_project_folderNumber)}AUDIO', level2_audio_folderNumber, 'MUSIC', 'AUDIO/MUSIC')
            case '6': createFolder_level2(f'{prefix(level1_project_folderNumber)}AUDIO', level2_audio_folderNumber, 'SOUNDTRACK', 'AUDIO/SOUNDTRACK')
            case '7': createFolder_level2(f'{prefix(level1_project_folderNumber)}AUDIO', level2_audio_folderNumber, 'VOICEOVER', 'AUDIO/VOICEOVER')
            case '8': createFolder_level2(f'{prefix(level1_project_folderNumber)}AUDIO', level2_audio_folderNumber, 'ADR', 'AUDIO/ADR')
            case _: createFolder_level2(f'{prefix(level1_project_folderNumber)}AUDIO', level2_audio_selection, string.upper(), f'AUDIO/{string.upper()}')
        level2_audio_folderNumber = level2_audio_folderNumber + 1



def createFolder_level2_export():
    clearConsole()
    print(f'''Select needed level 2 folders for {level1_folder_name}

    1) PREVIEWS
    2) MASTER
    3) DELIVERABLES

Type numbers to select or type custom folder names (separate by SPACE)''')
    
    level2_export_selection = input('$ ').split()
    level2_export_folderNumber = 1

    for string in level2_export_selection:
        match string:
            case '1': createFolder_level2(f'{prefix(level1_project_folderNumber)}EXPORT', level2_export_folderNumber, 'PREVIEWS', 'EXPORT/PREVIEWS')
            case '2': createFolder_level2(f'{prefix(level1_project_folderNumber)}EXPORT', level2_export_folderNumber, 'MASTER', 'EXPORT/MASTER')
            case '3': createFolder_level2(f'{prefix(level1_project_folderNumber)}EXPORT', level2_export_folderNumber, 'DELIVERABLES', 'EXPORT/DELIVERABLES')
            case _: createFolder_level2(f'{prefix(level1_project_folderNumber)}EXPORT', level2_export_folderNumber, string.upper(), f'EXPORT/{string.upper()}')
        level2_export_folderNumber = level2_export_folderNumber + 1



def createFolder_level2_documents():
    clearConsole()
    print(f'''Select needed level 2 folders for {level1_folder_name}
    
    1) NOTES
    2) MOODBOARD
    3) STORYBOARD
    4) SCRIPT
    5) PITCH
    6) LEGAL
    7) FINANCIAL

Type numbers to select or type custom folder names (separate by SPACE)''')
    
    level2_documents_selection = input('$ ').split()
    level2_documents_folderNumber = 1

    for string in level2_documents_selection:
        match string:
            case '1': createFolder_level2(f'{prefix(level1_project_folderNumber)}DOCUMENTS', level2_documents_folderNumber, 'NOTES', 'DOCUMENTS/NOTES')
            case '2': createFolder_level2(f'{prefix(level1_project_folderNumber)}DOCUMENTS', level2_documents_folderNumber, 'MOODBOARD', 'DOCUMENTS/MOODBOARD')
            case '3': createFolder_level2(f'{prefix(level1_project_folderNumber)}DOCUMENTS', level2_documents_folderNumber, 'STORYBOARD', 'DOCUMENTS/STORYBOARD')
            case '4': createFolder_level2(f'{prefix(level1_project_folderNumber)}DOCUMENTS', level2_documents_folderNumber, 'SCRIPT', 'DOCUMENTS/SCRIPT')
            case '5': createFolder_level2(f'{prefix(level1_project_folderNumber)}DOCUMENTS', level2_documents_folderNumber, 'PITCH', 'DOCUMENTS/PITCH')
            case '6': createFolder_level2(f'{prefix(level1_project_folderNumber)}DOCUMENTS', level2_documents_folderNumber, 'LEGAL', 'DOCUMENTS/LEGAL')
            case '7': createFolder_level2(f'{prefix(level1_project_folderNumber)}DOCUMENTS', level2_documents_folderNumber, 'FINANCIAL', 'DOCUMENTS/FINANCIAL')
            case _: createFolder_level2(f'{prefix(level1_project_folderNumber)}DOCUMENTS', level2_documents_folderNumber, string.upper(), f'DOCUMENTS/{string.upper()}')
        level2_documents_folderNumber = level2_documents_folderNumber + 1



def createFolder_level2_marketing():
    clearConsole()
    print(f'''Select needed level 2 folders for {level1_folder_name}
    
    1) POSTERS
    2) SOCIAL MEDIA
    3) TRAILERS
    
Type numbers to select or type custom folder names (separate by SPACE)''')
    
    level2_marketing_selection = input('$ ').split()
    level2_marketing_folderNumber = 1

    for string in level2_marketing_selection:
        match string:
            case '1': createFolder_level2(f'{prefix(level1_project_folderNumber)}MARKETING', level2_marketing_folderNumber, 'POSTERS', 'MARKETING/POSTERS')
            case '2': createFolder_level2(f'{prefix(level1_project_folderNumber)}MARKETING', level2_marketing_folderNumber, 'SOCIAL MEDIA', 'MARKETING/SOCIAL MEDIA')
            case '3': createFolder_level2(f'{prefix(level1_project_folderNumber)}MARKETING', level2_marketing_folderNumber, 'TRAILERS', 'MARKETING/TRAILERS')
            case _: createFolder_level2(f'{prefix(level1_project_folderNumber)}MARKETING', level2_marketing_folderNumber, string.upper(), f'MARKETING/{string.upper()}')
        level2_marketing_folderNumber = level2_marketing_folderNumber + 1



def createFolder_level2_custom(level2_custom_parent, relative_template_path):
    clearConsole()
    print(f'''Select needed level 2 folders for {level2_custom_parent}

Type custom folder names (separate by SPACE)''')
    
    level2_custom_selection = input('$ ').split()
    level2_custom_folderNumber = 1

    for string in level2_custom_selection:
        createFolder_level2(level2_custom_parent, level2_custom_folderNumber, string.upper(), relative_template_path)
        level2_custom_folderNumber = level2_custom_folderNumber + 1



def createFolder_level3(parent_level1:str, parent_level2:str, number:int, name:str, relative_template_path:str):
    level3_folder_name = f'{prefix(number)}{name}'
    level3_folder_directory = os.path.join(projectDirectory, parent_level1, parent_level2, level3_folder_name)

    os.mkdir(level3_folder_directory)
    if templates.check(relative_template_path) == True:
        templates.copy(relative_template_path, level3_folder_directory, level3_folder_name)



def createFolder_level3_raw():
    clearConsole()
    print(f'''Select needed level 3 folders for {level2_folder_name}

    1) A-CAM
    2) B-CAM
    3) C-CAM
    4) D-CAM
    99) SORT LATER

Type numbers to select or type custom folder names (separate by SPACE)''')
    
    level3_raw_selection = input('$ ').split()
    level3_raw_folderNumber = 1

    for string in level3_raw_selection:
        match string:
            case '1': createFolder_level3(f'{prefix(level1_project_folderNumber)}FOOTAGE', f'{prefix(level2_footage_folderNumber)}RAW', level3_raw_folderNumber, 'A-CAM', 'FOOTAGE/RAW/A-CAM')
            case '2': createFolder_level3(f'{prefix(level1_project_folderNumber)}FOOTAGE', f'{prefix(level2_footage_folderNumber)}RAW', level3_raw_folderNumber, 'B-CAM', 'FOOTAGE/RAW/B-CAM')
            case '3': createFolder_level3(f'{prefix(level1_project_folderNumber)}FOOTAGE', f'{prefix(level2_footage_folderNumber)}RAW', level3_raw_folderNumber, 'C-CAM', 'FOOTAGE/RAW/C-CAM')
            case '4': createFolder_level3(f'{prefix(level1_project_folderNumber)}FOOTAGE', f'{prefix(level2_footage_folderNumber)}RAW', level3_raw_folderNumber, 'D-CAM', 'FOOTAGE/RAW/D-CAM')
            case '99': createFolder_level3(f'{prefix(level1_project_folderNumber)}FOOTAGE', f'{prefix(level2_footage_folderNumber)}RAW', 99, 'SORT LATER', 'FOOTAGE/RAW/SORT LATER')
            case _: createFolder_level3(f'{prefix(level1_project_folderNumber)}FOOTAGE', f'{prefix(level2_footage_folderNumber)}RAW', level3_raw_folderNumber, string.upper(), f'FOOTAGE/RAW/{string.upper()}')
        level3_raw_folderNumber = level3_raw_folderNumber + 1



def createProxies():
    '''Ask user to make proxies & input codec and resolution. Transcode footage.'''

    clearConsole()
    print('Would you like to create proxies for RAW footage? (Y/n)\n(You can always do this later! See --help)')
    if input('$ ').lower() == 'n':
        print('Skipping proxy creation.')
    
    else:
        clearConsole()
        transcode.detect_defaults()
        print(f'Select codec (supported: h264, h264-nvidia, h264-amd, h264-intel, dnxhr, prores-proxy, prores-lt):')
        codec_selection = input('$ ').lower()
        if codec_selection:
            codec = codec_selection
        else:
            codec = transcode.detect_defaults('codec')

        clearConsole()
        print(f'Select resolution (ex. 1280x720):')
        resolution_selection = input('$ ').lower()
        if resolution_selection:
            resolution = resolution_selection
        else:
            resolution = transcode.detect_defaults('resolution')

        transcode.create_proxies(os.path.join(projectDirectory, proxies_footage), codec, resolution)


# Copyright (C) 2025  Andreas Delabie

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.



import sysconfig, os, json, typer



python_sitepackages = sysconfig.get_path('purelib')
app = typer.Typer(add_completion=False)

if not os.path.exists(f'{python_sitepackages}/initfilm/config.json'):
    with open(f'{python_sitepackages}/initfilm/config_default.json', 'r') as config_default:
        config_default = json.load(config_default)
        with open(f'{python_sitepackages}/initfilm/config.json', 'w') as config:
            config.seek(0)
            json.dump(config_default, config, indent=4)
            config.truncate()



def set(name:str, parameter:str, value:str):
    '''Change values in the config file.
    Args:
        name (str): Name of the config property you want to change. (ex. 'prefix' or 'templates')
        parameter (str): Name of the property parameter you want to change. (ex. 'number_style' or 'path')
        value (str): Value to change the property to. (ex. 'default' or 'dot')'''

    with open(f'{python_sitepackages}/initfilm/config.json', 'r+') as file:
        config = json.load(file)
        if name not in config:
            config[name] = {}  # Create section if missing
        config[name][parameter] = value
        file.seek(0)
        json.dump(config, file, indent=4)
        file.truncate()



def get(name:str, parameter:str) -> str:
    '''Get values from the config file.
    Args:
        name (str): Name of the config property you want to get. (ex. 'prefix' or 'templates')
        parameter (str): Name of the property parameter you want to get. (ex. 'number_style' or 'path')
    Returns:
        value (str): The value of the parameter you defined.'''

    with open(f'{python_sitepackages}/initfilm/config.json', 'r') as file:
        config = json.load(file)
    return config[name][parameter]     



@app.command()
def set_prefix_visibility(visibility:str):
    '''Set prefix visibility to 'visible' or 'hidden'.'''
    match input:
        case 'visible' | 'hidden':
            set('prefix', 'visibility', visibility)
        case _:
            print("Please specify if you want the prefix to be 'visible' or 'hidden'.")

@app.command()
def set_number_style(style:str):
    '''Set number style to 'default' or 'double'.'''
    match style:
        case 'default' | 'double':
            set('prefix', 'number_style', style)
        case _:
            print("Invalid number style! Valid options are 'default' or 'double'.")

@app.command()
def set_separator_style(style:str):
    '''Set separator style to 'dot', 'underscore', 'parenthesis' or 'space'.'''
    match style:
        case 'dot' | 'underscore' | 'parenthesis' | 'space':
            set('prefix', 'separator_style', style)
        case _:
            print("Invalid separator style! Valid options are 'dot', 'underscore', 'parenthesis' or 'space'.")

@app.command()
def set_templates_path(path:str):
    '''Set templates path (source path) to 'python' or custom path. ALWAYS USE FORWARD SLASHES!'''
    set('templates', 'path', path)

@app.command()
def set_proxy_codec(codec:str):
    '''Set default proxy codec to 'h264', 'h264-nvidia', 'h264-amd', 'h264-intel', 'dnxhr', 'prores-proxy' or 'prores-lt'.'''
    match codec:
        case 'h264' | 'h264-nvidia' | 'h264-amd' | 'h264-intel' | 'dnxhr' | 'prores-proxy' | 'prores-lt':
            set('proxies', 'default_codec', codec)
        case _:
            print("Invalid codec! Valid options are 'h264', 'h264-nvidia', 'h264-amd', 'h264-intel', 'dnxhr', 'prores-proxy' or 'prores-lt'.")

@app.command()
def set_proxy_resolution(resolution:str):
    '''Set default proxy resolution. (ex. '1280x720', '1920x1080', '3840x2160', ...).'''
    set('proxies', 'default_resolution', resolution)

@app.command()
def show():
    '''Print the current configuration.'''
    with open(f'{python_sitepackages}/initfilm/config.json', 'r') as file:
        config = json.dumps(json.load(file), indent=2)
    print(f'{python_sitepackages}/initfilm/config.json\n' + config)

@app.command()
def add_shortcut():
    '''Adds the Init-Film shortcut to the Windows context menu (right click menu).'''
    from . import shortcut
    shortcut.add()

@app.command()
def remove_shortcut():
    '''Removes the Init-Film shortcut from the Windows context menu (right click menu).'''
    from . import shortcut
    shortcut.remove()


# Copyright (C) 2025  Andreas Delabie

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.



import platform, subprocess



def detectOS() -> str:
    match platform.system():
        case 'Windows':
            return 'windows'
        case 'Darwin':
            return 'mac'
        case _:
            print("We haven't detected a Windows or MacOS operating system, so we suspect you are using Linux.\nWhat's your distro based on?")
            platform_linux = input('(debian/arch/fedora)$ ')
            if platform_linux.lower() not in ['debian', 'arch', 'fedora']:
                print("We need to know if your distro is based on Debian, Arch or Fedora to install FFmpeg correctly.\nIf you know what you're doing you can always install FFmpeg manually through your distro's package manager!")
                exit(1)
            return platform_linux.lower()



def install_windows():
    try:
        subprocess.call('winget install ffmpeg', shell=True)
    except Exception as e:
        print(f"An error occurred while trying to install FFmpeg on Windows: {e}")



def install_mac():
    try:
        print('Are you using Apple Silicon (M1,M2,M3,...) or Intel?')
        match input('(silicon/intel)$ '):
            case 'silicon':
                subprocess.call('curl -O https://www.osxexperts.net/ffmpeg711arm.zip', shell=True)
                subprocess.call('sudo unzip ffmpeg711arm.zip -d /opt/ffmpeg', shell=True)
            case 'intel':
                subprocess.call('curl -O https://www.osxexperts.net/ffmpeg71intel.zip', shell=True)
                subprocess.call('sudo unzip ffmpeg71intel.zip -d /opt/ffmpeg', shell=True)
        subprocess.call('echo "/opt/ffmpeg" | sudo tee -a /etc/paths', shell=True)
    except Exception as e:
        print(f"An error occurred while trying to install FFmpeg on MacOS: {e}")



def install_linux(distro: str):
    try:
        match distro:
            case 'debian':
                subprocess.call('sudo apt install ffmpeg', shell=True)
            case 'arch':
                subprocess.call('sudo pacman -S ffmpeg', shell=True)
            case 'fedora':
                subprocess.call('sudo dnf install ffmpeg', shell=True)
    except Exception as e:
        print(f"An error occurred while trying to install FFmpeg on Linux: {e}")



def main():
    try:
        subprocess.call('ffmpeg -version', stdout=subprocess.DEVNULL)
        print("FFmpeg is already installed.")
    except:
        system_os = detectOS()
        match system_os:
            case 'windows':
                install_windows()
            case 'mac':
                install_mac()
            case 'debian':
                install_linux('debian')
            case 'arch':
                install_linux('arch')
            case 'fedora':
                install_linux('fedora')
        print("FFmpeg installation complete. You can now use it with Init-Film.")